/*******************************************************************************
 *	BDebug.h	BEAM Debug classes
 *			T.Barnaby,	BEAM Ltd,	2007-10-17
 *******************************************************************************
 */
#ifndef BDEBUG_H
#define BDEBUG_H

class BDebugBacktrace {
public:
			BDebugBacktrace();
			~BDebugBacktrace();

	void		dumpBacktraceStdout(char* comment);
	int		dumpBacktraceFile(char* fileName, char* comment);
	void		dumpBacktraceSyslog(char* comment);
	void		dumpBacktrace(char* strBuf, int strBufLen, char* comment);
private:
};

#endif
