#ifdef ZAP	
	{
	int     data;
	
	// Reset Local bus
	data = 1;
	ADMXRC2_Write(ofpgaCard,ADMXRC2_IOWIDTH_32,ADMXRC2_IOADAPTER,0x28,&data,4);
	data = 0;
	ADMXRC2_Write(ofpgaCard,ADMXRC2_IOWIDTH_32,ADMXRC2_IOADAPTER,0x28,&data,4);
	sleep(1);
	}
#endif
	
	printf("FPGA: LOCKED: %x\n", ofpga[LOCKED]);
#ifdef ZAP
	printf("Reset DCM\n");
	ofpga[LOCKED] = 1;
	usleep(100000);
	ofpga[LOCKED] = 0;
	usleep(2000000);
#endif

#ifdef ZAP
	printf("Reset SDRAM\n");
	ofpga[LOCKED] = 2;
	usleep(100000);
	ofpga[LOCKED] = 0;
	usleep(2000000);
#endif
