/*******************************************************************************
 *	GenBDataPy.h	Generate Data output
 *			T.Barnaby,	BEAM Ltd,	12/9/03
 *******************************************************************************
 */
#ifndef GENBDATAPY_HH
#define GENBDATAPY_HH

#include <Gen.h>

class GenBDataPy : public Gen {
public:
			GenBDataPy();
			~GenBDataPy();
	BError		produce(Node* n, BString fileName);
	BString		getTypeName(Node* n, int data = 0);
private:
	BError		produceHeaderImp();
	BError		produceTrailerImp();
	void		produceImp(Node* n);

	BString		ofileName;
	FileIndent	ofileImp;
	BString		omodule;
};

#endif
