/*******************************************************************************
 *	TestcaseT.h	Produced by Bidl
 *******************************************************************************
 */

#ifndef TESTCASET_H
#define TESTCASET_H 1

#include <BString.h>
#include <BList.h>
#include <BArray.h>
#include <TestcaseS.h>

using namespace Appliance;

class LightServer : public Appliance::LightService {
public:
		LightServer(BoapServer& server, BString name);
	BError setLevel(Int32 level);
};

class SwitchServer : public Appliance::SwitchService {
public:
		SwitchServer(BoapServer& server, BString name);
	BError swtchChanged(Int32 level);
};

class TvServer : public Appliance::TvService {
public:
		TvServer(BoapServer& server, BString name);
	BError setChannel(Int32 channel);
};

class TvRecorderServer : public Appliance::TvRecorderService {
public:
		TvRecorderServer(BoapServer& server, BString name);
	BError setChannel(Int32 channel);
};

#endif
