/*******************************************************************************
 *	Client.cc	HomeNet RPC protocol test client
 *			T.Barnaby,	BEAM Ltd,	9/5/03
 *******************************************************************************
 */
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <Boap.h>

int main(int argc, char** argv){
	BoapClientObject	client("telControl");
	BError			err;

	while(1){
		err = client.connectService("telControl");
		printf("Connection: %s\n", err.getString().retStr());
		sleep(1);
	}
	
	return 0;
}
