/*
* Title:	AppGui.h
* Author:	M.Thomas BEAM Ltd
* Date:		2005-10-11
*
* Contents:
*
* Mod Rec:
*
*/

#ifndef	APPGUI_H
#define APPGUI_H	1

#include <qapp.h>
#include <qmainwindow.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qaccel.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <BString.h>
#include <BList.h>

class AppGui: public QMainWindow {
	Q_OBJECT
public:
				AppGui();
	virtual			~AppGui();
	virtual void		init(QString title);
	virtual void		initMenubar();
	virtual void		initMain();
	virtual void		initStatus();
	virtual void		initToolbar();
	

	// Helper functions
	virtual QAction*	addAction(QPopupMenu* menu, QString name, QString menuName, QString accelKey, QString memberFunc, QString tip);
public slots:
	virtual void		slotQuit();

private:
	BList<QPopupMenu*>	omenus;
	BList<QAction*>		oactions;
    	QToolBar *toolbar;
	
};
#endif
