/*******************************************************************************
 *	Main.h			Tms process overal definitions
 *				T.Barnaby,	BEAM Ltd,	2007-02-07
 *******************************************************************************
 */
#ifndef Main_H
#define Main_H	1

#include <Config.h>

extern Config	config;								///< The configuration
extern int	realTime;							///< Is a Realtime process

// System Error information
const int	errOk			= 0;	///< No error, all is Ok
const int	errMisc			= 1;	///< Generic error
const int	errWarning		= 2;	///< Generic warning
const int	errInit			= 3;	///< Initialisation Error
const int	errConfig		= 4;	///< Config File Error
const int	errParam		= 5;	///< Parameter Error

#endif
