/*******************************************************************************
 *	Config.h	Config File Access
 *			T.Barnaby,	BEAM Ltd,	25/7/03
 *******************************************************************************
 */
#ifndef CONFIG_H
#define	CONFIG_H

#include <BEntry.h>
#include <BError.h>
#include <BMutex.h>

/// This class implements the configuration file access
class Config : private BEntryFile {
public:
	int		open(BString filename);
	int		read();
	int		write();
	BString		findValue(BString name);
	int		setValue(BString name, BString value);
private:
	BMutex		olock;
};
#endif
