/*******************************************************************************
 *	Control.h	Control process
 *			T.Barnaby,	BEAM Ltd,	2007-05-23
 *******************************************************************************
 */
#ifndef Control_H
#define Control_H	1

#include <stdint.h>
#include <BError.h>
#include <Config.h>
#include <NetworkHttp.h>

using namespace std;

/// The main system control class
class Control {
public:
			Control();
			~Control();

	// Main functionality	
	BError		init();				///< Initialise the system
	void		run();				///< Start the system running

	Config		oconfig;
	NetworkHttp	ohttpServer;

};

#endif
